@@ -7,11 +7,11 @@ Created on Thu Apr  8 20:25:55 2021
 
 
 #Import required modules
-import csv
-import os
+import csv                            
 import numpy as np
 import datetime
 import dateutil.parser as parser
+import os
 
     
 
@@ -53,8 +53,18 @@ class Filter:
         """
         return str(self.cases)
         
-    
-    def by_gender(self,frequency='absolute',decimals=3):
+    def values(self):
+        """Return raw ndarray.
+        
+        Returns
+        -------
+        cases : ndarray
+            Returns the object's cases as an ndarray.
+        
+        """
+        return self.cases    
+        
+    def by_gender(self,frequency:str='absolute',decimals:int=3):
         """Converts the Cases to a representation by gender.
         
         Parameters
@@ -74,7 +84,7 @@ class Filter:
         if (frequency.lower() == 'absolute' or result_all == 0):
             return_data = {}
             for index, key in enumerate(_GENDERS.keys()):
-                return_data[self.case_description+'_'+key] = result[index]
+                return_data[self.case_description+'_'+key] = round(result[index],1)
             return return_data
         elif (frequency.lower() == 'relative' and result_all != 0):
             return_data = {}
@@ -83,7 +93,7 @@ class Filter:
             return return_data 
             
         
-    def by_age(self,frequency='absolute',decimals=3):
+    def by_age(self,frequency:str='absolute',decimals:int=3):
         """Converts the Cases to a representation by age.
         
         Parameters
@@ -103,7 +113,7 @@ class Filter:
         if (frequency.lower() == 'absolute' or result_all == 0):
             return_data = {}
             for index, key in enumerate(_AGE_GROUPS.keys()):
-                return_data[self.case_description+'_'+key] = result[index]
+                return_data[self.case_description+'_'+key] = round(result[index],1)
             return return_data
         elif (frequency.lower() == 'relative' and result_all != 0):
             return_data = {}
@@ -112,7 +122,7 @@ class Filter:
             return return_data
             
      
-    def by_ageandgender(self,frequency='absolute',decimals=3):
+    def by_ageandgender(self,frequency:str='absolute',decimals:int=3):
         """Converts the Cases to a representation by age and gender.
         
         Parameters
@@ -132,19 +142,21 @@ class Filter:
         for age_index,age in enumerate(_AGE_GROUPS.keys()):
             for gender_index,gender in enumerate(_GENDERS.keys()):
                 if (frequency.lower() == 'absolute' or result_all == 0):
-                    return_data[self.case_description+'_'+age+'_'+gender] = self.cases[age_index][gender_index]
+                    return_data[self.case_description+'_'+age+'_'+gender] = round(self.cases[age_index][gender_index],1)
                 elif (frequency.lower() == 'relative' and result_all != 0):
                     return_data[self.case_description+'_'+age+'_'+gender] = round(self.cases[age_index][gender_index]/result_all,decimals)
         return return_data
     
     
-    def by_cases(self):
+    def by_cases(self, raw:bool=False, decimals:int=1):
         """Converts the Cases to an absolute number.
         
         Parameters
         ----------
         frequency : str, optional
-            relative or absolute frewuency for the output.
+            relative or absolute frequency for the output.
+        raw : bool, optional
+            if true the raw values are returned.
         decimals : int, optional
             number of decimal places.
 
@@ -152,10 +164,15 @@ class Filter:
         -------
         return_data : dict
             a dictionary with the absolute number of cases.
+        return_data : float
+            a float with the absolute number of cases.
         """
-        return_data = {}
-        return_data[self.case_description] = self.cases.sum(axis=0).sum(axis=0)
-        return return_data
+        if(raw==True):
+            return round(self.cases.sum(axis=0).sum(axis=0),decimals)
+        else:    
+            return_data = {}
+            return_data[self.case_description] = round(self.cases.sum(axis=0).sum(axis=0),decimals)
+            return return_data
     
     
 
@@ -256,23 +273,27 @@ class covid_cases:
         self._loaded_rki_cases = np.load(path)
     
         
-    def kumFälle(self, date='2021-01-01 00:00:00', region_id='0', date_type='Meldedatum'):
+    def cumCases(self, date, region_id='0', date_type='Meldedatum'):
         """Return the cumulated Covid19 cases for the given day and region.
         
         Parameters
         ----------
-        date : str, optional
-            The date. The default is '2021-01-01 00:00:00'.
+        date : str in iso format, datetime.date obj, datetime.datetime obj
+            The desired date.
         region_id : str, optional
-            Region. The default is '0'.
+            ID of the desired Region. The default is '0'.
         date_type : str, optional
             The type of date. The default is 'Meldedatum'.
-    
+        
         Returns
         -------
-        Object of class Case : Case object
-            Returns an object of the class Case.
+        Object of class Filter : Filter object
+            Returns an object of the class Filter.
         """
+        if(type(date)==datetime.date):
+            date = str(datetime.datetime.combine(date, datetime.time()))   
+        elif(type(date)==datetime.datetime): 
+            date = str(date)    
         covid_cases = self._loaded_rki_cases
         dates = _load_dates()
         datetype_index = _DATE_TYPES[date_type]
@@ -290,27 +311,31 @@ class covid_cases:
         elif (int(region_id) > 1000):
             lk_id = _load_lk_ids()[region_id.zfill(5)]
             result = covid_cases[lk_id,0:dates[date]+1,0,datetype_index].sum(axis=0)
-            
+        
         return Filter(result, 'kumFälle_{}'.format(date_type))
     
     
-    def kumTodesfälle(self, date='2021-01-01 00:00:00', region_id='0', date_type='Meldedatum'):
+    def cumDeaths(self, date, region_id='0', date_type='Meldedatum'):
         """Return the cumulated Covid19 deaths for the given day and region.
         
         Parameters
         ----------
-        date : str, optional
-            The date. The default is '2021-01-01 00:00:00'.
+        date : str in iso format, datetime.date obj, datetime.datetime obj
+            The desired date.
         region_id : str, optional
-            Region. The default is '0'.
+            ID of the desired Region. The default is '0'.
         date_type : str, optional
             The type of date. The default is 'Meldedatum'.
-    
+        
         Returns
         -------
-        Object of class Case : Case object
-            Returns an object of the class Case.
+        Object of class Filter : Filter object
+            Returns an object of the class Filter.
         """
+        if(type(date)==datetime.date):
+            date = str(datetime.datetime.combine(date, datetime.time()))   
+        elif(type(date)==datetime.datetime): 
+            date = str(date)
         covid_cases = self._loaded_rki_cases
         dates = _load_dates()
         datetype_index = _DATE_TYPES[date_type]
@@ -328,28 +353,32 @@ class covid_cases:
         elif (int(region_id) > 1000):
             lk_id = _load_lk_ids()[region_id.zfill(5)]
             result = covid_cases[lk_id,0:dates[date]+1,1,datetype_index].sum(axis=0)
-            
+        
         return Filter(result, 'kumTodesfälle_{}'.format(date_type))
 
 
-    def neueFälle(self, date='2021-01-01 00:00:00', region_id='0', date_type='Meldedatum'):
+    def newCases(self, date, region_id='0', date_type='Meldedatum'):
         """Return the new Covid19 cases for the given day and region.
         
         Parameters
         ----------
-        date : str, optional
-            The date. The default is '2021-01-01 00:00:00'.
+        date : str in iso format, datetime.date obj, datetime.datetime obj
+            The desired date.
         region_id : str, optional
-            Region. The default is '0'.
+            ID of the desired Region. The default is '0'.
         date_type : str, optional
             The type of date. The default is 'Meldedatum'.
-    
+        
         Returns
         -------
-        Object of class Case : Case object
-            Returns an object of the class Case.
+        Object of class Filter : Filter object
+            Returns an object of the class Filter.
         """
-        covid_cases = self.loaded_rki_cases
+        if(type(date)==datetime.date):
+            date = str(datetime.datetime.combine(date, datetime.time()))   
+        elif(type(date)==datetime.datetime): 
+            date = str(date)
+        covid_cases = self._loaded_rki_cases
         dates = _load_dates()
         datetype_index = _DATE_TYPES[date_type]
         
@@ -366,28 +395,32 @@ class covid_cases:
         elif (int(region_id) > 1000):
             lk_id = _load_lk_ids()[region_id.zfill(5)]
             result = covid_cases[lk_id,dates[date],0,datetype_index]
-            
+        
         return Filter(result, 'neueFälle_{}'.format(date_type)) 
 
     
-    def neueTodesfälle(self, date='2021-01-01 00:00:00', region_id='0', date_type='Meldedatum'):
+    def newDeaths(self, date, region_id='0', date_type='Meldedatum'):
         """Return the new Covid19 desths for the given day and region.
         
         Parameters
         ----------
-        date : str, optional
-            The date. The default is '2021-01-01 00:00:00'.
+        date : str in iso format, datetime.date obj, datetime.datetime obj
+            The desired date.
         region_id : str, optional
-            Region. The default is '0'.
+            ID of the desired Region. The default is '0'.
         date_type : str, optional
             The type of date. The default is 'Meldedatum'.
     
         Returns
         -------
-        Object of class Case : Case object
-            Returns an object of the class Case.
+        Object of class Filter : Filter object
+            Returns an object of the class Filter.
         """
-        covid_cases = self.loaded_rki_cases
+        if(type(date)==datetime.date):
+            date = str(datetime.datetime.combine(date, datetime.time()))   
+        elif(type(date)==datetime.datetime): 
+            date = str(date)
+        covid_cases = self._loaded_rki_cases
         dates = _load_dates()
         datetype_index = _DATE_TYPES[date_type]
         
@@ -404,19 +437,19 @@ class covid_cases:
         elif (int(region_id) > 1000):
             lk_id = _load_lk_ids()[region_id.zfill(5)]
             result = covid_cases[lk_id,dates[date],1,datetype_index]
-            
+        
         return Filter(result, 'neueTodesfälle_{}'.format(date_type))
 
 
-    def neueFälleZeitraum(self, date='2021-01-01 00:00:00', region_id='0', date_type='Meldedatum', timespan=1):
+    def newCasesTimespan(self, date, region_id='0', date_type='Meldedatum', timespan=1):
         """Return the new Covid19 cases for the given day and region.
         
         Parameters
         ----------
-        date : str, optional
-            The date. The default is '2021-01-01 00:00:00'.
+        date : str in iso format, datetime.date obj, datetime.datetime obj
+            The desired date.
         region_id : str, optional
-            Region. The default is '0'.
+            ID of the desired Region. The default is '0'.
         date_type : str, optional
             The type of date. The default is 'Meldedatum'.
         timespan : int
@@ -424,10 +457,10 @@ class covid_cases:
             
         Returns
         -------
-        Object of class Case : Case object
-            Returns an object of the class Case.
+        Object of class Filter : Filter object
+            Returns an object of the class Filter.
         """
-        covid_cases = self.loaded_rki_cases
+        covid_cases = self._loaded_rki_cases
         dates = _load_dates()
         datetype_index = _DATE_TYPES[date_type]
         
@@ -450,15 +483,15 @@ class covid_cases:
         return Filter(result, 'neueFälle_{}Tage_{}'.format(timespan,date_type))
 
 
-    def neueTodesfälleZeitraum(self, date='2021-01-01 00:00:00', region_id='0', date_type='Meldedatum', timespan=1):
+    def newDeathsTimespan(self, date, region_id='0', date_type='Meldedatum', timespan=1):
         """Return the new Covid19 cases for the given day and region.
         
         Parameters
         ----------
-        date : str, optional
-            The date. The default is '2021-01-01 00:00:00'.
+        date : str in iso format, datetime.date obj, datetime.datetime obj
+            The desired date.
         region_id : str, optional
-            Region. The default is '0'.
+            ID of the desired Region. The default is '0'.
         date_type : str, optional
             The type of date. The default is 'Meldedatum'.
         timespan : int
@@ -466,10 +499,10 @@ class covid_cases:
             
         Returns
         -------
-        Object of class Case : Case object
-            Returns an object of the class Case.
+        Object of class Filter : Filter object
+            Returns an object of the class Filter.
         """
-        covid_cases = self.loaded_rki_cases
+        covid_cases = self._loaded_rki_cases
         dates = _load_dates()
         datetype_index = _DATE_TYPES[date_type]
         
@@ -492,7 +525,190 @@ class covid_cases:
         return Filter(result, 'neueTodesfälle_{}Tage_{}'.format(timespan,date_type))        
 
 
- 
+    def activeCases(self, date, region_id='0', date_type='Meldedatum', days_infectious=14):
+        """Return the active Covid19 cases for the given day and region.
+        
+        Parameters
+        ----------
+        date : str in iso format, datetime.date obj, datetime.datetime obj
+            The desired date.
+        region_id : str, optional
+            ID of the desired Region. The default is '0'.
+        date_type : str, optional
+            The type of date. The default is 'Meldedatum'.
+        days_infectious : int
+            The number of days an case is considered active after the transmission of the infection.
+            
+        Returns
+        -------
+        Object of class Filter : Filter object
+            Returns an object of the class Filter.
+        """
+        if(type(date)==datetime.date):
+            date = str(datetime.datetime.combine(date, datetime.time()))   
+        elif(type(date)==datetime.datetime): 
+            date = str(date)
+        covid_cases = self._loaded_rki_cases
+        dates = _load_dates()
+        datetype_index = _DATE_TYPES[date_type]
+        
+        start_date = parser.isoparse(date)-datetime.timedelta(days=days_infectious)
+        
+        if (int(region_id) == 0):
+            result = covid_cases[0:,dates[str(start_date)]:dates[date],0,datetype_index].sum(axis=0).sum(axis=0)
+            
+        elif (int(region_id) >= 1 and int(region_id) <= 16):
+            lk_ids = _load_lk_ids()
+            result = np.zeros((7,3),dtype=int)
+            for value,index in list(lk_ids.items()):
+                if (value[0:2]==region_id.zfill(2)):
+                    result = np.add(result,covid_cases[index,dates[str(start_date)]:dates[date],0,datetype_index].sum(axis=0))
+                    
+        elif (int(region_id) > 1000):
+            lk_id = _load_lk_ids()[region_id.zfill(5)]
+            result = covid_cases[lk_id,dates[str(start_date)]:dates[date],0,datetype_index].sum(axis=0)
+            
+        return Filter(result, 'aktiveFälle_{}'.format(date_type))
+      
+        
+    def sevenDayCaserate(self, date, region_id='0', date_type='Meldedatum'):
+        """Return the new Covid19 cases for the last 7 days from the given date.
+        
+        Parameters
+        ----------
+        date : str in iso format, datetime.date obj, datetime.datetime obj
+            The desired date.
+        region_id : str, optional
+            ID of the desired Region. The default is '0'.
+        date_type : str, optional
+            The type of date. The default is 'Meldedatum'.
+            
+        Returns
+        -------
+        Object of class Filter : Filter object
+            Returns an object of the class Filter.
+        """
+        if(type(date)==datetime.date):
+            date = str(datetime.datetime.combine(date, datetime.time()))   
+        elif(type(date)==datetime.datetime): 
+            date = str(date)
+        covid_cases = self._loaded_rki_cases
+        dates = _load_dates()
+        datetype_index = _DATE_TYPES[date_type]
+        
+        start_date = parser.isoparse(date)-datetime.timedelta(days=7)
+        
+        if (int(region_id) == 0):
+            result = covid_cases[0:,dates[str(start_date)]:dates[date],0,datetype_index].sum(axis=0).sum(axis=0)
+            
+        elif (int(region_id) >= 1 and int(region_id) <= 16):
+            lk_ids = _load_lk_ids()
+            result = np.zeros((7,3),dtype=int)
+            for value,index in list(lk_ids.items()):
+                if (value[0:2]==region_id.zfill(2)):
+                    result = np.add(result,covid_cases[index,dates[str(start_date)]:dates[date],0,datetype_index].sum(axis=0))
+                    
+        elif (int(region_id) > 1000):
+            lk_id = _load_lk_ids()[region_id.zfill(5)]
+            result = covid_cases[lk_id,dates[str(start_date)]:dates[date],0,datetype_index].sum(axis=0)
+            
+        return Filter(result, '7-TageFallzahl_{}'.format(date_type)) 
+    
+    
+    def sevenDayIncidence(self, date, region_id='0', date_type='Meldedatum'):
+        """Return the Covid19 cases per 100 000 residents for the last 7 days from the given date.
+        
+        Parameters
+        ----------
+        date : str in iso format, datetime.date obj, datetime.datetime obj
+            The desired date.
+        region_id : str, optional
+            ID of the desired Region. The default is '0'.
+        date_type : str, optional
+            The type of date. The default is 'Meldedatum'.
+            
+        Returns
+        -------
+        Object of class Filter : Filter object
+            Returns an object of the class Filter.
+        """
+        if(type(date)==datetime.date):
+            date = str(datetime.datetime.combine(date, datetime.time()))   
+        elif(type(date)==datetime.datetime): 
+            date = str(date)
+        covid_cases = self._loaded_rki_cases
+        dates = _load_dates()
+        datetype_index = _DATE_TYPES[date_type]
+        dividor = _get_population(region_id)*(1/100000)
+        
+        start_date = parser.isoparse(date)-datetime.timedelta(days=7)
+        
+        if (int(region_id) == 0):
+            result = covid_cases[0:,dates[str(start_date)]:dates[date],0,datetype_index].sum(axis=0).sum(axis=0)
+            result = np.divide(result,dividor)
+            
+        elif (int(region_id) >= 1 and int(region_id) <= 16):
+            lk_ids = _load_lk_ids()
+            result = np.zeros((7,3),dtype=int)
+            for value,index in list(lk_ids.items()):
+                if (value[0:2]==region_id.zfill(2)):
+                    result = np.add(result,covid_cases[index,dates[str(start_date)]:dates[date],0,datetype_index].sum(axis=0))
+                    result = np.divide(result,dividor)
+                    
+        elif (int(region_id) > 1000):
+            lk_id = _load_lk_ids()[region_id.zfill(5)]
+            result = covid_cases[lk_id,dates[str(start_date)]:dates[date],0,datetype_index].sum(axis=0)
+            result = np.divide(result,dividor)
+            
+        return Filter(result, '7-TageInzidenz_{}'.format(date_type))
+    
+    
+    def deathRate(self, date, region_id='0', days_infectious=14):
+        """Return the death rate for the given date.
+        
+        Parameters
+        ----------
+        date : str in iso format, datetime.date obj, datetime.datetime obj
+            The desired date.
+        region_id : str, optional
+            ID of the desired Region. The default is '0'.
+        days_infectious : int
+            The number of days an case is considered active after the transmission of the infection.
+            
+        Returns
+        -------
+        Object of class Filter : Filter object
+            Returns an object of the class Filter.
+        """
+        active_cases = self.activeCases(date=date, region_id=region_id, date_type='Refdatum', days_infectious=days_infectious)
+        new_deaths = self.newDeaths(date=date, region_id=region_id, date_type='Meldedatum')
+        result = np.divide(new_deaths, active_cases, where=active_cases != 0)
+        return Filter(result, 'Todesrate')
+    
+    
+    def deathRate(self, date, region_id='0', days_infectious=14):
+        """Return the death rate for the given date.
+        
+        Parameters
+        ----------
+        date : str in iso format, datetime.date obj, datetime.datetime obj
+            The desired date.
+        region_id : str, optional
+            ID of the desired Region. The default is '0'.
+        days_infectious : int
+            The number of days an case is considered active after the transmission of the infection.
+            
+        Returns
+        -------
+        Object of class Filter : Filter object
+            Returns an object of the class Filter.
+        """
+        active_cases = self.activeCases(date=date, region_id=region_id, date_type='Refdatum', days_infectious=days_infectious).values()
+        new_deaths = self.newDeaths(date=date, region_id=region_id, date_type='Meldedatum').values()
+        result = np.divide(new_deaths, active_cases, where=active_cases != 0)
+        return Filter(result, 'Todesrate')
+    
+    
 #internal function to create dates dict   
 def _load_dates():
     """Return a dict with the dates from the start_date to the current date.
@@ -523,12 +739,42 @@ def _load_lk_ids():
         Dictionary containing Landkreise and indicies.
     """
     lk_ids = {}
+    #lk_names = {}
     path = os.path.join(os.path.dirname(__file__), 'data/Landkreis_id.csv')
     csv_file = open(path, mode='r', encoding='UTF-8')
     csv_reader = csv.reader(csv_file, delimiter=',', quotechar='"')
     next(csv_reader)
     for index,row in enumerate(csv_reader):
         lk_ids[row[0].zfill(5)] = index
+        #lk_names[row[0].zfill(5)] = row[1]
     csv_file.close()
     return lk_ids
+
+
+#internal function to get population
+def _get_population(region_id):
+    """Return the population for a given region.
     
+    Returns
+    -------
+    population : int
+        population for the given region_id.
+    """
+    if (int(region_id) >= 0 and int(region_id) <= 16):
+        path = os.path.join(os.path.dirname(__file__), 'data/Bundesland_id.csv')
+        csv_file = open(path, mode='r', encoding='UTF-8')
+        csv_reader = csv.reader(csv_file, delimiter=',', quotechar='"')
+        next(csv_reader)
+        for index,row in enumerate(csv_reader):
+            if (row[0] == region_id.zfill(2)):
+                return int(row[2])
+                        
+    elif (int(region_id) > 1000):
+        path = os.path.join(os.path.dirname(__file__), 'data/Landkreis_id.csv')
+        csv_file = open(path, mode='r', encoding='UTF-8')
+        csv_reader = csv.reader(csv_file, delimiter=',', quotechar='"')
+        next(csv_reader)
+        for index,row in enumerate(csv_reader):
+            if (row[0] == region_id.zfill(5)):
+                return int(row[2])
+    return 0    
